#!/bin/bash
#
# Copyright (C) 2023 MOXA Inc. All rights reserved.
# This software is distributed under the terms of the MOXA SOFTWARE NOTICE.
# See the file LICENSE for details.
#
# Authors:
#       2023  Wilson YS Huang  <wilsonys.huang@moxa.com>

source "${INSTL_SCRIPT_DIR}/lib/util.sh"
source "${INSTL_SCRIPT_DIR}/lib/logger.sh"
source "${INSTL_SCRIPT_DIR}/lib/driver.sh"
log::init "${INSTL_LOG_FILE}" "${INSTL_LOG_LEVEL}"

set -e

REPO_NAME="moxa-it87-wdt-driver"
SRC_PATH_PREFIX="${INSTL_TOP_DIR}/src/${REPO_NAME}"

main() {
        local src_tag
        local src_ver
        src_tag="$1"
        src_ver="$2"

        log::info "Installing ${REPO_NAME} (${src_ver})"

        # RedHat/CentOS will load it87_wdt.ko.xz first after driver install.
        if [[ "${LINUX_DISTRO_ID}" == "rhel" ]] || [[ "${LINUX_DISTRO_ID}" == "centos" ]]; then
                local drv_path
                drv_path=$(modinfo -n it87_wdt 2>/dev/null || true)
                if [[ "${drv_path##*/}" == "it87_wdt.ko.xz" ]]; then
                        rm -f "${drv_path}"
                fi
        fi

        {
                pushd "${SRC_PATH_PREFIX}-${src_tag}" >/dev/null

                make clean
                make install
                make load

                popd
        } >>"${INSTL_LOG_FILE}" 2>&1

        log::info " Setup Watchdog Service"

        cat <<EOF >/lib/modules-load.d/it87_wdt.conf
it87_wdt
EOF

        cat <<EOF >/lib/modprobe.d/it87_wdt-option.conf
# timeout:Watchdog timeout in seconds, default=60 (int)
# nowayout:Watchdog cannot be stopped once started, default=0 (bool)
options it87_wdt nowayout=1 timeout=$CFG_IT87_WDT_TIMEOUT ldn_reset=1
EOF

        {
                local ignore_wdt_drv
                local ignored_list

                ignore_wdt_drv="iTCO_wdt iTCO_vendor_support mei_wdt wdat_wdt"
                ignored_list=$(drv::add_blacklist "$ignore_wdt_drv" "blacklist-wdt.conf")

                if [[ "${LINUX_DISTRO_ID}" == "rhel" ]] || [[ "${LINUX_DISTRO_ID}" == "centos" ]]; then
                        if ! util::is_empty_str "$ignored_list"; then
                                cat <<EOF >/etc/dracut.conf.d/omit-wdt.conf
omit_dracutmodules+=" $ignore_wdt_drv "
omit_drivers+=" $ignore_wdt_drv "
EOF
                                dracut -c /etc/dracut.conf.d/omit-wdt.conf -f /boot/initramfs-"$(uname -r)".img "$(uname -r)"

                                for mod in ${ignored_list}; do
                                        grubby --args "$mod.blacklist=1 rd.driver.blacklist=$mod" --update-kernel ALL
                                        sed -i "/^KDUMP_COMMANDLINE_APPEND=/s/\"$/ rd.driver.blacklist=$mod\"/" /etc/sysconfig/kdump
                                done

                                kdumpctl restart
                                mkdumprd -f /boot/initramfs-"$(uname -r)"kdump.img
                        fi
                fi

        } >>"${INSTL_LOG_FILE}" 2>&1

        case "${LINUX_DISTRO_ID}" in
        "debian" | "ubuntu")
                {
                        apt-get update
                        apt-get install watchdog -y
                } >>"${INSTL_LOG_FILE}" 2>&1

                ;;
        "rhel" | "centos")
                {
                        yum makecache
                        yum install watchdog -y
                } >>"${INSTL_LOG_FILE}" 2>&1
                ;;
        esac

        cat <<EOF >/etc/default/watchdog
# Start watchdog at boot time? 0 or 1
run_watchdog=1
# Start wd_keepalive after stopping watchdog? 0 or 1
run_wd_keepalive=0
# Load module before starting watchdog
watchdog_module="it87_wdt"
# Specify additional watchdog options here (see manpage).
EOF

        {
                sed -i "/^#watchdog-device/s/^#//g" /etc/watchdog.conf
                systemctl enable watchdog
                systemctl start watchdog
        } >>"${INSTL_LOG_FILE}" 2>&1
}

main "$@"
